import java.io.*;
import java.net.*;

public class DaytimeClient {
    private String server_name;
    private int server_port;

    public DaytimeClient(String server, int port) throws IOException {
        server_name = server;
        server_port = port;
    }

    public DaytimeClient(String server) throws IOException {
        server_name = server;
        // la porta TCP 13 è la porta standard del servizio daytime
        server_port = 13;
    }

    public String getDaytime() throws SocketTimeoutException, IOException {
        InputStream stream;
        String answer = "";
        String fragment;
        int n;
        byte [] buffer = new byte[1024];
        Socket client_socket = new Socket();
        InetSocketAddress server_address = new InetSocketAddress(server_name, server_port);

        client_socket.setSoTimeout(1000); // 1000ms = 1s
        // richiesta di connessione al server (attesa massima 1s)
        client_socket.connect(server_address, 1000); // 1000ms = 1s
        // stream di input per ricezione dati dal server
        stream = client_socket.getInputStream();
        // ciclo di lettura dei dati ricevuti dal server nello stream di input fino alla chiusura da parte del server
        while ((n = stream.read(buffer)) != -1) {
            fragment = new String(buffer, 0, n, "ISO-8859-1");
            answer = answer + fragment;
        }
        // chiusura dello stream di ricezione dei dati e del socket di connessione al server
        stream.close();
        //client_socket.shutdownInput();
        client_socket.close();
        return answer;
    }

    public static void main(String args[]) {
        String server;
        int port;
        String daytime;
        DaytimeClient client;
        
        if (args.length != 3) {
            server = "127.0.0.1"; // localhost
            port = 13; // porta TCP standard servizio daytime
        }
        else {
            server = args[0];
            port = Integer.parseInt(args[1]);
        }
        try {
            client = new DaytimeClient(server, port);
            daytime = client.getDaytime();
            System.out.println(daytime);
        }
        catch (SocketTimeoutException exception) {
            System.err.println("Nessuna risposta dal server!");
        }
        catch (IOException exception) {
            System.err.println("Errore generico di comunicazione!");
        }
    }
}
