
import java.net.*;
import java.io.*;

public class UDPclient {
    private DatagramSocket socket;

    public UDPclient() throws SocketException {
        socket = new DatagramSocket();
        socket.setSoTimeout(1000); // 1000ms = 1s
    }

    public void close_socket() {
        socket.close();
    }
    
    public String sendAndReceive( String request, String host, int port) throws UnknownHostException, IOException, SocketTimeoutException {
        byte[] buffer;
        DatagramPacket datagram;
        String answer;
        // indirizzo IP del destinatario del datagram
        InetAddress address = InetAddress.getByName(host);
        
        // verifica chiusura socket
        if ( socket.isClosed()) {
            throw new IOException();
        }
        // trasformazione in array di byte della stringa
        buffer = request.getBytes("UTF-8");
        // costruzione datagram di richiesta
        datagram = new DatagramPacket( buffer, buffer.length, address, port);
        socket.send(datagram); // trasmissione datagram di richiesta
        // attesa ricezione datagram di richiesta (tempo massimo di attesa: 1s)
        socket.receive(datagram);
        // verifica indirizzo/porta provenienza datagram di risposta
        if ( datagram.getAddress().equals(address) && datagram.getPort() == port) {
            answer = new String( datagram.getData(), 0, datagram.getLength(), "ISO-8859-1");
        }
        else {
            throw new SocketTimeoutException();
        }
        return answer;
    }
    
    public static void main(String args[]) {
        String IP_address;
        int UDP_port;
        String request, answer;
        UDPclient client;

        if ( args.length != 3) {
            IP_address = "127.0.0.1";
            UDP_port = 7;
            request = "Ciao!";
        }
        else {
            IP_address = args[0];
            UDP_port = Integer.parseInt(args[1]);
            request = args[2];
        }
        try {
            client = new UDPclient();
            answer = client.sendAndReceive(request, IP_address, UDP_port);
            System.out.println("Ricevuto in risposta: " + answer);
            client.close_socket();
        }
        catch (SocketException exception) {
            System.err.println("Errore creazione socket!");
        }
        catch (UnknownHostException exception) {
            System.err.println("Indirizzo IP errato!");
        }
        catch (SocketTimeoutException exception) {
            System.err.println("Nessuna risposta dal server!");
        }
        catch (IOException exception) {
            System.err.println("Errore generico di comunicazione!");
        }
    }
}
